try:
    import requests 
    from bs4 import BeautifulSoup
    from fake_headers import Headers
    import urllib3
    import re
#if module is not available
except Exception as ex:
    print(ex)
    exit()

class Chapter_list:
    def __init__(self,manga):
        """
        instantiate class with given manga name
        """
        self.manga = manga
        self.URL = f"http://mangareader.cc/manga/{self.URLify(self.manga)}"

    def URLify(self,manga):
        return "-".join(manga.split(" "))
    
    def remove_trails(self,string):
        string = re.sub(r"\s", "", string)
        string = re.sub("([a-z])([A-Z])","\g<1> \g<2>",string)
        return string


    def get_chapter_list(self):
        """
        returns list of all chapter's from given manga/
        """
        ua = Headers(headers = False) #change headers
        
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)         #hiding the warning
        
        response = requests.get(self.URL,headers = ua.generate(),verify = False)           #sending a request and storing the response inside response var
        
        if response.status_code >= 400 and response.status_code < 500:     #server error
            print("Server Error!\nTry again later")
        if response.status_code == 200:
            soup = BeautifulSoup(response.content,"html.parser")
    
            unorder_lists = soup.findAll("ul")
            all_spans = unorder_lists[2].findChildren('span',{'class' : 'leftoff'})
            all_chapters = list(reversed(list(map(self.remove_trails,[span.text for span in all_spans]))))                
            
            return all_chapters


    def get_links(self):
        """
        returns list of all chapter's link from https://mangareader.cc/
        """
        ua = Headers(headers = False) #change headers
        
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)         #hiding the warning
        
        response = requests.get(self.URL,headers = ua.generate(),verify = False)           #sending a request and storing the response inside response var
        
        if response.status_code >= 400 and response.status_code < 500:     #if server error
            print("Server Error\nTry again later")
        if response.status_code >= 200 and response.status_code < 300:
            soup = BeautifulSoup(response.content,"html.parser")

            unorder_list = soup.findAll("ul")[2]
            all_hyperlink_tags = unorder_list.findChildren('a') 
            all_hrefs = list(reversed([hyperlink.get('href') for hyperlink in all_hyperlink_tags]))

            return all_hrefs



try:
    from .chapter_reader import Chapter_reader
    import os
except Exception as ex:
    print(ex)


class Manga(Chapter_reader):
    def __init__(self,manga):
        self.manga = manga
        self.URL = f"http://www.mangareader.cc/manga/{self.URLify(self.manga)}"
    
   


try:
    import requests 
    from bs4 import BeautifulSoup
    from .chapter_list import Chapter_list
    from fake_headers import Headers
    import re
    import time
    import os
    import random
    import urllib3
except Exception as ex:
    print(ex)
    exit()

class Chapter_reader(Chapter_list):  
    '''
    base class is Chapter_list, so,
    - URLify() method is inherited from Chapter_list
    - get_links() method is inherited as well
    '''
    def __init__(self, manga):   #constructor needs manga by default
        """
        instantiate chapter_reader class 
        """
        self.manga = self.URLify(manga)
        self.URL = f"http://mangareader.cc/manga/{self.URLify(self.manga)}"

    def get_image_links(self,chapter_number):
        """returns all image links present for manga chapter"""
        
        URLS = self.get_links()             #all chapters, method from Chapter_list class(base class)
   
        headers = Headers(headers=False).generate()        
        
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning) #disable warning
        
        response = requests.get(URLS[int(chapter_number)],headers = headers,verify = False)            #chapter number
        
        if response.status_code < 500 and response.status_code >= 400:   #if server error 
            print("Server Error!\nTry Again later")
            exit()

        if response.status_code >= 200 and response.status_code < 300:
            #if response is success

            soup = BeautifulSoup(response.content,"html.parser") #make bs4 object with response's content and parser is html.parser   
            
            paragraph = soup.find("p",{"id":"arraydata"})

            all_image_hrefs = paragraph.text.split(',')

            return all_image_hrefs
            
    
    def create_folder(self,chapter_number):
        #if folder with name of given manga exists already
        if os.path.isdir(os.path.join(os.getcwd(), f'{self.manga}')):
                # changing current directory to given manga name folder
                os.chdir(os.path.join(os.getcwd(), f'{self.manga}'))
                
                # if folder with name as given chapter also exists in the folder
                if os.path.isdir(os.path.join(os.getcwd(), f'{chapter_number}')):
                    # just set the current working directory to this one, and exit function
                    os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
                    
                    return "Starting download ..."   #exit
                else:
                    # create the chapter folder with given manga name
                    os.mkdir(f'{chapter_number}')
                    # change current working directory to newly created folder
                    os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
                    return "Starting download ..."
        #if folder with name of given manga does not exist
        else:

            # making folder with same manga name
            os.mkdir(f'{self.manga}')
            # changing directory to that above created folder
            os.chdir(os.path.join(os.getcwd(), f'{self.manga}'))
            # ceating /manga/chapter_number folder
            os.mkdir(f'{chapter_number}')
            # ceating chapter folder inside current folder
            os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
            return "Starting download ..."



    def download(self,URL,file_name):
        """expects image links,downloads them, if download was succesful,return True else False"""
        headers = Headers().generate()
        
        image_extension = URL.split(".")[-1]
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning) 
        response = requests.get(URL, stream=True,headers = headers,verify = False)
                
        if response.status_code == 200:
            with open(f'{self.manga} - {file_name}.{image_extension}', 'wb') as file:
                file.write(response.content)    
                return True
        return False
    

    def download_chapter(self,chapter_number,file_location = os.getcwd()):
        '''download the chapter'''
        try:
            #if given directory is invalid than download in current directory
            if not os.path.exists(file_location):
                file_location = os.getcwd()

            print(f"Searching for {self.manga}")
            
            time.sleep(random.randint(1, 5))
            
            print(f"Succesfully fetched all images on the server...\nCreating Folder {self.manga}...")
            
            # folder creating process
            
            os.chdir(file_location)

            self.create_folder(chapter_number) #change directory to where image has to been downloaded
            
            # loops through all image links, send a response and if response is success,
            # write that response.content as binary as images

            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)         #hiding the warning
            
            img_links = self.get_image_links(chapter_number)
            
            print(f'{len(img_links)+1} Pages to download...')
            
            for i in range(len(img_links)):
                download_result = self.download(img_links[i],f"{self.manga} - Page {i+1}")
                if download_result:
                        print(f"{self.manga} - Chapter : {chapter_number} Page :{i+1} downloaded...")
                        print(f'Remaining {len(img_links)-i}')    
                        time.sleep(random.randint(5, 10))
                else:
                    print(f"Could not able to download {i+1} page")

            return f"Successfully downloaded {self.manga} - {chapter_number}\nEnjoy the manga!:)\nBye"
        
        except IndexError:
            print(f"{chapter_number} does not exist!")
        except KeyboardInterrupt:
            print("Bye")
            exit()
        except Exception as ex:
            print(ex)



try:
    import requests
    import urllib3
    from fake_headers import Headers
    from .chapter_list import Chapter_list 
    from bs4 import BeautifulSoup
    import re
    import os
    import time
    import random
except Exception as ex:
    print(ex)

class Chapter_reader(Chapter_list):
    def __init__(self,manga):
        self.manga = self.URLify(manga)
        self.URL = f"https://mangapark.net/manga/{self.URLify(self.manga)}/"
    def get_image_links(self,chapter_number):
        """returns a list of image link for a given chapter"""
        #make a request to server and scrap all present chapter for manga
        manga_chapters = self.get_links()  #calling from base class Chapter_list
    
        #we get a list where chapter is listed in descending order, so we'll reverse it to ascending order
        manga_chapters.reverse()
        
        #store chapter that has to be scrapped
        chapter_to_scrap = manga_chapters[int(chapter_number)-1]
        
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning) #disable warning
        
        #send request to server with given chapter URL
        response = requests.get(chapter_to_scrap,headers = Headers().generate(),verify=False)

        if response.status_code >= 400 and response.status_code < 500:
            #if response status code is 400-499
            print("Server error,Please try again later")
            exit()
        if response.status_code >= 200 and response.status_code < 300:
            #if successful response 
            soup = BeautifulSoup(response.content,"html.parser")    
            
            all_tags = re.findall('"u":".*?"',soup.prettify())   #using regex find all url present in js code
            
        
            all_urls = [f"https:"+url.split(":")[2].replace('"','').replace("\\","") for url in all_tags]
            
            return all_urls      
    
    def create_folder(self,chapter_number):
        #if folder with name of given manga exists already
        if os.path.isdir(os.path.join(os.getcwd(), f'{self.manga}')):
                # changing current directory to given manga name folder
                os.chdir(os.path.join(os.getcwd(), f'{self.manga}'))
                
                # if folder with name as given chapter also exists in the folder
                if os.path.isdir(os.path.join(os.getcwd(), f'{chapter_number}')):
                    # just set the current working directory to this one, and exit function
                    os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
                    
                    return "Starting download ..."   #exit
                else:
                    # create the chapter folder with given manga name
                    os.mkdir(f'{chapter_number}')
                    # change current working directory to newly created folder
                    os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
                    return "Starting download ..."
        #if folder with name of given manga does not exist
        else:

            # making folder with same manga name
            os.mkdir(f'{self.manga}')
            # changing directory to that above created folder
            os.chdir(os.path.join(os.getcwd(), f'{self.manga}'))
            # ceating /manga/chapter_number folder
            os.mkdir(f'{chapter_number}')
            # ceating chapter folder inside current folder
            os.chdir(os.path.join(os.getcwd(), f'{chapter_number}'))
            return "Starting download ..."



    def download(self,URL,file_name):
        """expects image links,downloads them, if download was succesful,return True else False"""
        headers = Headers().generate()
        
        image_extension = URL.split("?")[0].split(".")[-1]
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning) 
        response = requests.get(URL, stream=True,headers = headers,verify = False)
                
        if response.status_code == 200:
            with open(f'{self.manga} - {file_name}.{image_extension}', 'wb') as file:
                file.write(response.content)    
                return True
        return False
    

    def download_chapter(self,chapter_number,file_location = os.getcwd()):
        '''download the chapter'''
        try:
            if not os.path.exists(file_location):
                file_location = os.getcwd()

            print(f"Searching for {self.manga}")
            
            time.sleep(random.randint(1, 5))
            
            print(f"Succesfully fetched all images on the server...\nCreating Folder {self.manga}...")
            
            # folder creating process
            
            os.chdir(file_location)

            self.create_folder(chapter_number)
            
            # loops through all image links, send a response and if response is success,
            # write that response.content as binary as images'''
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)         #hiding the warning
            
            img_links = self.get_image_links(chapter_number)
            print(f'{len(img_links)+1} Pages to download...')
            
            for i in range(len(img_links)):
                res = self.download(img_links[i],f"{self.manga} - Page {i+1}")
                if res:
                        print(f"{self.manga} - Chapter : {chapter_number} Page :{i+1} downloaded...")
                        print(f'Remaining {len(img_links)-i}')    
                        time.sleep(random.randint(5, 10))
                else:
                    print(f"Could not able to download {i+1} page")

            return f"Successfully downloaded {self.manga} - {chapter_number}\nEnjoy the manga!:)\nBye"
        
        except IndexError:
            print(f"{chapter_number} does not exist!")
        except KeyboardInterrupt:
            print("Bye")
            exit()
        except Exception as ex:
            print(ex)







try:
    import requests
    from bs4 import BeautifulSoup
    from fake_headers import Headers
    import urllib3
except Exception as ex:
    print(ex)

class Chapter_list:
    def __init__(self,manga):
        self.manga = manga
        self.URL = f"https://mangapark.net/manga/{self.URLify(self.manga)}/"
        
    def URLify(self,manga):
        """replaces space with '-' """
        return "-".join(manga.split(" "))

    def __replace_ending(self,sentence, old, new):
        if sentence.endswith(old):
            return sentence[:-len(old)] + new
        return sentence
    
    def get_chapter_list(self):
        """returns list of all chapter available for given manga"""

        user_agent = Headers(os="win").generate() #fake user agent    
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning) 
        
        #send request and store them to response var
        response = requests.get(self.URL,headers = user_agent,verify = False)

        if response.status_code < 500 and response.status_code >= 400:
            #if server error occured
            print("Server error")
            exit()
        
        if response.status_code >= 200 and response.status_code < 300:
            soup = BeautifulSoup(response.content,"html.parser")
            
            #find table tag with id named as "stream_3"
            table = soup.find("div",{"id" : "stream_3"})
            
            if table is None:
                print("Cannot find the manga on MangaPanda")
                exit()
            #find div with class name "d-none" inside table tag
            all_div_tags = table.find_all("div",{"class" : "d-none"})

            #extract text between those div tags,replace \n escape character,":" and spaces as well
            all_texts = [text.get_text().strip().replace("\n","").strip().replace(":","").strip() for text in all_div_tags]
            #if div tags inside table was not found
            if len(all_texts) == 0:
                #find all anchor tag
                all_anchor_tags = table.find_all("a",{"class" : "visited"})
                all_texts = [text.get_text() for text in all_anchor_tags]   #extract text between anchor tags

            #we get list in descending order,so reverse it to make it ascending order
            all_texts.reverse()
            
            return [f"{self.manga} {index} : {value}" for index,value in enumerate(all_texts,start=1)]


    def get_links(self):
        user_agent = Headers().generate()    
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)    
        response = requests.get(self.URL,headers = user_agent,verify = False)

        if response.status_code < 500 and response.status_code > 400:
            print("Server error")
            exit()
        
        if response.status_code >= 200 and response.status_code < 300:
            soup = BeautifulSoup(response.content,"html.parser")

            #find table that contains all chapters 
            table = soup.find("div",{"id" : "stream_3"})
            #all anchor tags inside tables
            all_anchor_tags = table.find_all("a",{"class" : "ml-1 visited ch"})
            #extract all href values from anchor tags
            all_hrefs = [f"https://mangapark.net{self.__replace_ending(anchor['href'],'/1','')}" for anchor in all_anchor_tags]

            return all_hrefs




import setuptools

with open("README.md","r") as file:
    long_description = file.read()

setuptools.setup(
    name = "manga_down",
    version = "0.1.2",
    author = "Shaikh Sajid",
    author_email = "shaikhsajid3732@gmail.com",
    description = "Python package to download manga available on mangareader and mangapanda",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/shaikhsajid1111/manga-down/",
    keywords = "web-scraping manga-down manga-download download manga-downloader requests bs4 beautifulsoup mangreader mangapanda",
    packages = setuptools.find_packages(),
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X", 
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP"
        

    ],
    python_requires = ">=3.6.9",
    install_requires=[
        'bs4==0.0.1',
        'requests==2.22.0',
        'fake-headers==1.0.2'
    ])
